#include "stdafx.h"
#include "Canvas.h"

Canvas::Canvas(HDC hdc,int x,int y){
	m_size.x = x;m_size.y = y;
	m_hdc = ::CreateCompatibleDC(hdc);
	m_bmp = ::CreateCompatibleBitmap(hdc,x,y);
	::DeleteObject(reinterpret_cast<HBITMAP>(::SelectObject(m_hdc,m_bmp)));

	m_pen = ::CreatePen(PS_SOLID,10,RGB(0,0,0));
	m_oldobj = ::SelectObject(m_hdc,reinterpret_cast<HGDIOBJ>(m_pen));

	m_font = ::CreateFont(10,0,0,0,FW_REGULAR,FALSE,FALSE,FALSE,
		DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,FIXED_PITCH |FF_MODERN ,
		_T("lr SVbN"));
	m_oldfont = ::SelectObject(m_hdc,reinterpret_cast<HGDIOBJ>(m_font));

	m_fillbrush = ::CreateSolidBrush(RGB(255,255,255));

	RECT rc;rc.left = 0;rc.right = x;rc.top = 0;rc.bottom = y;
	::FillRect(m_hdc,&rc,reinterpret_cast<HBRUSH>(::GetStockObject(WHITE_BRUSH)));
	m_tmphdc = ::CreateCompatibleDC(hdc);
	m_redoundostate = m_redoundo.end();
	AddUndoChain();
}

Canvas::~Canvas(void){
	::SelectObject(m_hdc,m_oldobj);
	::DeleteObject(m_pen);
	::SelectObject(m_hdc,m_oldfont);
	::DeleteObject(m_font);
	::DeleteObject(m_fillbrush);
	::DeleteDC(m_hdc);
	::DeleteObject(m_bmp);
	for(std::list<HBITMAP>::iterator i = m_redoundo.begin() ; i != m_redoundo.end() ; ++i)::DeleteObject(*i);
	::DeleteObject(m_tmphdc);
}


void Canvas::SetLineStyle(ablib::window::rgb rgb,int width,ablib::window::Window::linetype style){
	m_pen = ::CreatePen(style,width,RGB(rgb.red,rgb.green,rgb.blue));
	::DeleteObject(::SelectObject(m_hdc,reinterpret_cast<HGDIOBJ>(m_pen)));
}

void Canvas::DrawLine(ablib::window::Point start,ablib::window::Point end){
	::MoveToEx(m_hdc,start.x,start.y,NULL);
	::LineTo(m_hdc,end.x,end.y);
}

void Canvas::SetFont(const TCHAR *fontname, int fontsize, int red, int green, int blue){
	m_fontsize = fontsize;m_fontname = fontname;
	m_fontrgb = ablib::window::rgb(red,green,blue);
	m_font = ::CreateFont(fontsize,0,0,0,FW_REGULAR,FALSE,FALSE,FALSE,
		DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,FIXED_PITCH |FF_MODERN ,
		fontname);
	::SetTextColor(m_hdc,RGB(red,green,blue));
	::DeleteObject(::SelectObject(m_hdc,reinterpret_cast<HGDIOBJ>(m_font)));
}

void Canvas::WriteText(ablib::window::Point pt,const TCHAR *str){
	::TextOut(m_hdc,pt.x,pt.y,str,::lstrlen(str));
}

void Canvas::SetFillColor(ablib::window::rgb rgb){
	m_fillcolor = rgb;
	::DeleteObject(m_fillbrush);
	m_fillbrush = ::CreateSolidBrush(RGB(rgb.red,rgb.green,rgb.blue));
}

void Canvas::FillRect(ablib::window::Rect rc){
	RECT r;r.top = rc.top;r.left = rc.left;r.right = rc.right;r.bottom = rc.bottom;
	::FillRect(m_hdc,&r,m_fillbrush);
}

void Canvas::Copy(HDC hdc,ablib::window::Rect &rc){
	::BitBlt(hdc,rc.left,rc.top,rc.right - rc.left,rc.bottom - rc.top,m_hdc,rc.left,rc.top,SRCCOPY);
}

void Canvas::AddUndoChain(){
	HBITMAP bitmap = ::CreateCompatibleBitmap(m_hdc,m_size.x,m_size.y);
	HGDIOBJ oldobj = ::SelectObject(m_tmphdc,reinterpret_cast<HGDIOBJ>(bitmap));
	::BitBlt(m_tmphdc,0,0,m_size.x,m_size.y,m_hdc,0,0,SRCCOPY);
	::SelectObject(m_tmphdc,oldobj);
	if(m_redoundostate != m_redoundo.end())++m_redoundostate;
	m_redoundo.erase(m_redoundostate,m_redoundo.end());
	m_redoundo.push_back(bitmap);
	if(m_redoundo.size() > 21){
		::DeleteObject(*m_redoundo.begin());
		m_redoundo.erase(m_redoundo.begin());
	}
	m_redoundostate = m_redoundo.end();
	--m_redoundostate;
}

bool Canvas::Undo(){
	if(m_redoundostate == m_redoundo.begin())return false;
	--m_redoundostate;
	HGDIOBJ oldobj = ::SelectObject(m_tmphdc,*m_redoundostate);
	::BitBlt(m_hdc,0,0,m_size.x,m_size.y,m_tmphdc,0,0,SRCCOPY);
	::SelectObject(m_tmphdc,oldobj);
	return true;
}

bool Canvas::Redo(){
	if(m_redoundostate == m_redoundo.end())return false;
	++m_redoundostate;
	if(m_redoundostate == m_redoundo.end()){--m_redoundostate;return false;}
	HGDIOBJ oldobj = ::SelectObject(m_tmphdc,*m_redoundostate);
	::BitBlt(m_hdc,0,0,m_size.x,m_size.y,m_tmphdc,0,0,SRCCOPY);
	::SelectObject(m_tmphdc,oldobj);
	return true;
}
