#include "stdafx.h"
#include <window/Graphic.h>
#include "MainWnd.h"
#include "resource.h"
#include "Ruby.h"
#include "Setting.h"

extern HINSTANCE g_appint;
extern Ruby g_Ruby;
extern Setting g_Setting;
extern MainWnd g_MainWnd;

MainWnd::MainWnd() : BaseWnd(_T("AbenoriDeskDraw"),WS_POPUP,g_appint,true),m_tasktray(NULL),m_moshook(NULL),m_canvas(NULL){}

bool MainWnd::Create(){
	bool rv = (BaseWnd::Create(NULL,::LoadCursor(NULL,IDC_ARROW)) != NULL);
	if(rv){
		m_tasktray = new ablib::window::TaskTray(GetWindow());
		m_tasktray->Set(::LoadIcon(g_appint,(LPCSTR)IDI_TASK),WM_TASKMESSAGE,_T("DeskDraw"));

		::SetWindowLong(GetWindow()->GetHWND(),GWL_EXSTYLE,(::GetWindowLong(GetWindow()->GetHWND(),GWL_EXSTYLE) | WS_EX_LAYERED | WS_EX_NOACTIVATE | WS_EX_TOOLWINDOW) & ~WS_EX_TRANSPARENT);
//		::SetWindowLong(GetWindow()->GetHWND(),GWL_EXSTYLE,(::GetWindowLong(GetWindow()->GetHWND(),GWL_EXSTYLE) |  WS_EX_NOACTIVATE | WS_EX_TOOLWINDOW) & ~WS_EX_TRANSPARENT);
		::SetLayeredWindowAttributes(GetWindow()->GetHWND(),RGB(255,255,255),0,LWA_COLORKEY);
		::SetWindowPos(GetWindow()->GetHWND(),HWND_TOPMOST,0,0,0,0,SWP_NOSIZE | SWP_NOMOVE);
		GetWindow()->ShowWindow(ablib::window::Wnd::sw_maximize);

		g_Ruby.Load(g_Setting.GetScriptPath());
	}
	return rv;
}

void MainWnd::SwitchMode(mode m){
	m_mode = m;
	switch(m){
	case mode_none:
		if(m_moshook){::UnhookWindowsHookEx(m_moshook);m_moshook = NULL;}
		if(m_keyhook){::UnhookWindowsHookEx(m_keyhook);m_keyhook = NULL;}
		return;
	case mode_draw:
		if(!m_moshook)m_moshook = ::SetWindowsHookEx(WH_MOUSE_LL,sLowLevelMouseProc,g_appint,0);
		if(!m_keyhook)m_keyhook = ::SetWindowsHookEx(WH_KEYBOARD_LL,LowLevelKeyboardProc,g_appint,0);
		return;
	default:
		return;
	}
}

LRESULT CALLBACK MainWnd::LowLevelKeyboardProc(int nCode,WPARAM wParam,LPARAM lParam){
	if(nCode != HC_ACTION)return ::CallNextHookEx(g_MainWnd.m_keyhook,nCode,wParam,lParam);
	KBDLLHOOKSTRUCT *kbd = reinterpret_cast<KBDLLHOOKSTRUCT*>(lParam);
	if(wParam == WM_SYSKEYDOWN && kbd->vkCode == VK_F4 && ::GetKeyState(VK_MENU) < 0){
		g_MainWnd.GetWindow()->SendMsg(WM_DESTROY,0,0);
		return TRUE;
	}
	if(wParam == WM_KEYDOWN)return g_Ruby.Event_OnKeyDown(kbd->vkCode) ? TRUE : ::CallNextHookEx(g_MainWnd.m_keyhook,nCode,wParam,lParam);
	else if(wParam == WM_KEYUP)return g_Ruby.Event_OnKeyUp(kbd->vkCode) ? TRUE : ::CallNextHookEx(g_MainWnd.m_keyhook,nCode,wParam,lParam);
	else return ::CallNextHookEx(g_MainWnd.m_keyhook,nCode,wParam,lParam);
}


LRESULT CALLBACK MainWnd::sLowLevelMouseProc(int nCode, WPARAM wParam, LPARAM lParam){
	return g_MainWnd.LowLevelMouseProc(nCode,wParam,lParam);
}

LRESULT CALLBACK MainWnd::LowLevelMouseProc(int nCode, WPARAM wParam, LPARAM lParam){
	if(nCode != HC_ACTION || m_mode != mode_draw)return ::CallNextHookEx(m_moshook,nCode,wParam,lParam);
	MSLLHOOKSTRUCT *mos = reinterpret_cast<MSLLHOOKSTRUCT*>(lParam);
	switch(wParam){
	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_RBUTTONDOWN:
		::PostMessage(GetWindow()->GetHWND(),wParam,0,MAKELPARAM(mos->pt.x,mos->pt.y));
		return TRUE;
	case WM_MOUSEMOVE:
		::PostMessage(GetWindow()->GetHWND(),wParam,0,MAKELPARAM(mos->pt.x,mos->pt.y));
		return FALSE;
	default:
		return ::CallNextHookEx(m_moshook,nCode,wParam,lParam);
	}
}


bool MainWnd::OnPaint(ablib::window::Window *wnd){
	PAINTSTRUCT ps;
	HDC hdc = ::BeginPaint(wnd->GetHWND(),&ps);
	if(m_canvas)m_canvas->Copy(hdc,ablib::window::Rect(ps.rcPaint));
	else ::FillRect(hdc,&(ps.rcPaint),reinterpret_cast<HBRUSH>(::GetStockObject(WHITE_BRUSH)));
	::EndPaint(wnd->GetHWND(),&ps);
	return true;
}

bool MainWnd::OnDestroy(ablib::window::Window *wnd){
	if(m_keyhook)::UnhookWindowsHookEx(m_keyhook);
	if(m_moshook)::UnhookWindowsHookEx(m_moshook);
	return true;
}


bool MainWnd::OnLButtonDown(ablib::window::Window *wnd,ablib::window::Mouse *mos,bool isDClick){
	if(m_mode != mode_draw)return false;
	ablib::window::Point pos = mos->GetPos();
	return g_Ruby.Event_OnLButtonDown(pos.x,pos.y,isDClick);
}

bool MainWnd::OnLButtonUp(ablib::window::Window *wnd,ablib::window::Mouse *mos){
	if(m_mode != mode_draw)return false;
	ablib::window::Point pos = mos->GetPos();
	return g_Ruby.Event_OnLButtonUp(pos.x,pos.y);
}

bool MainWnd::OnMouseMove(ablib::window::Window *wnd,ablib::window::Mouse *mos,int keyFlags){
	if(m_mode != mode_draw)return false;
	ablib::window::Point pos = mos->GetPos();
	return g_Ruby.Event_OnMouseMove(pos.x,pos.y);
}

bool MainWnd::OnRButtonDown(ablib::window::Window *wnd,ablib::window::Mouse *mos,bool isDClick){
	if(m_mode != mode_draw)return false;
	ablib::window::Point pos = mos->GetPos();
	return g_Ruby.Event_OnRButtonDown(pos.x,pos.y,isDClick);
}

bool MainWnd::PreTranslateMessage(ablib::window::Window *wnd,UINT uMsg,WPARAM wParam,LPARAM lParam,LRESULT &rv){
	switch(uMsg){
	case WM_TASKMESSAGE:
		switch(lParam){
		case WM_RBUTTONDOWN:
		{
			ablib::window::Menu *m = wnd->GetMenu();
			m->AddRootMenu(1,_T("`惂[hON"));
			m->AddRootMenu(2,_T("XNvg̓ǂݍ"));
			m->AddRootMenu(3,_T("I"));
			POINT pt;
			::GetCursorPos(&pt);
			switch(m->CreatePopupMenu(ablib::window::Point(pt),ablib::window::Menu::pos_leftbottom)){
			case 0:
				return false;
			case 1:
				SwitchMode(mode_draw);
				return true;
			case 2:
			{
				ablib::window::CommonDlg dlg(wnd);
				ablib::string rv;
				if(dlg.OpenFileDlg(rv,_T("RubyXNvg\0*.rb\0SẴt@C\0*.*\0"),
					_T("ǂݍރXNvgIłD"),g_Setting.GetScriptPath())){
						g_Ruby.Load(rv.c_str());
				}
				return true;
			}
			case 3:
				wnd->Destroy();
				return true;
			default:
				return false;
			}
		}
		default:
			return false;
		}
	default:
		return false;
	}
}